C**************************************************************************
C MAP_STEEL_TTT_TO_CCT
C
C SUBROUTINE TO CALCULATE CCT FROM TTT
C USING SIMULATED WELD COOLING CURVES
C
C CALCULATIONS INDICATE THAT CCT CURVES CALCULATED USING
C LINEAR COOLING ARE VERY SIMILAR.
C
C THIS SUBROUTINE COULD EASILY BE MODIFIED TO USE LINEAR COOLING.
C
C COPYRIGHT S J JONES & H K D H BHADESHIA
C           DEPT OF MATERIALS SCIENCE & METALLURGY
C           UNIVERSITY OF CAMBRIDGE
C           UNITED KINGDOM
C           JANUARY 1996
C
C SUBROUTINE TO CALCULATE CCT FROM TTT
C USING SIMULATED WELD COOLING CURVES
C
C
C WARNING INPUT TTT MUST START AT HIGHEST TEMPERATURE - NOT THE LOWEST
C
C IINFORM=1 INPUT TTT CURVE POINTS WITH 20 DEGC INTERVALS (MUCG46.F)
C IINFORM=2 INPUT TTT CURVE POINTS WITH 1 DEGC INTERVALS (OTHER SMALL
C           INTERVALS SHOULD BE OK - BUT USE WITH CARE)
C
C INPUT TTT POINTS ARE AT 20 DEGC INTERVALS - AS GENERATED BY THE TTT
C MODEL "MAP_STEEL_MUCG46" BY HKDB BHADESHIA ET AL. THEREFORE NEED TO 
C INTERPOLATE A TTT CURVE WITH 1 DEGC INTERVALS.
C
C TTT TO CCT CONVERSION PROCEDURE:
C OUTER LOOP - ARRANGE COOLING RATE "MODEL" - REDUCE CALCULTION.  
C MIDDLE LOOP - VARY COOLING RATE.
C INNER LOOPWILL - SCHEILS ADDITIVE REACTION RULE -  FOR COOLING CURVE.
C   
C TMTTT,TITTT=INPUT TTT CURVE, IPAIRS=NO POINTS
C TTEMP,TTIME=CALCULATED (FULL - INTERPOLATION) TTT CURVE, ITTTPT=NO POINTS
C TMCCT,TICCT=CALCULATED CCT CURVE, ICCTPT=NO POINTS
C
C**************************************************************************
C
        SUBROUTINE MAP_STEEL_TTT_TO_CCT(IINFORM,IPAIRS,THEQU,PAE3,MS,
     &  ISWITCH,TITTT,TMTTT,ITTTPT,TTIME,TTEMP,ICCTPT,TICCT,TMCCT)
C
        IMPLICIT NONE
C
        DOUBLE PRECISION TITTT(1000),TMTTT(1000),
     &  TMCCT(300),TICCT(300),TTEMP(1000),TTIME(1000)
C
        DOUBLE PRECISION THEQU,PAE3,MS,TITTT,TMTTT,TTIME,
     &  TTEMP,TICCT,TMCCT,TEMP,PRETEMP
        DOUBLE PRECISION ANUM,BNUM,CONS,AVAL,BVAL,DELTA,
     &  C1,C2,A2,A3,AFAC,BFAC,STARTEFFHT,NEW1EFFHT,PRE2EFFHT,
     &  NEW2EFFHT,EFFHT,SUM,PREEFFHT,TINTV,FRAC,PRESUM,
     &  RTEMP,CONA,CONB,REAST
        INTEGER IINFORM,IPAIRS,ISWITCH,ITTTPT,ICCTPT,
     &  IBEGIN,IELOOP,IENDL,IPLONE,I35
        INTEGER J5,J7,J10,J20,J30,J40,J60,J70,J80
C
C SET OUTPUT ARRAYS TO ZERO - ON 2ND AND SUBSEQUENT SUBROUTINE CALLS
C
	IF (ISWITCH.EQ.1) THEN

	DO 5 J5=1,300
	  TICCT(J5)=0.0
	  TMCCT(J5)=0.0
5	CONTINUE

	DO 7 J7=1,1000
	  TITTT(J7)=0.0
	  TMTTT(J7)=0.0
7	CONTINUE

	DO 10 J10=1,1000
	  TTEMP(J10)=0.0
	  TTIME(J10)=0.0
10	CONTINUE

         PRETEMP=0.0
         TEMP=0.0

	ENDIF


      IF (IINFORM.EQ.1) THEN

C INPUT TTT POINTS ARE AT 20 DEGC INTERVALS - AS GENERATED BY THE TTT
C MODEL "MUCG46.F" BY HKDB BHADESHIA ET AL. THEREFORE NEED TO INTERPOLATE
C A TTT CURVE WITH 1 DEGC INTERVALS.

C GENERATE FULL TTT ARRAY FROM MUCG46.f ARRAY
C BY INTERPOLATION BETWEEN 20 DEGC POINTS:

      IPLONE=IPAIRS-2

      DO 20 J20=0,IPLONE 
        TTEMP(J20*20+1)=TMTTT(J20+1)
        TTIME(J20*20+1)=TITTT(J20+1)
        TTEMP(J20*20+21)=TMTTT(J20+2)
        TTIME(J20*20+21)=TITTT(J20+2)
        ANUM=(TTIME(J20*20+1)-TTIME(J20*20+21))
        BNUM=(TTEMP(J20*20+1)-TTEMP(J20*20+21))
        CONS=ANUM/BNUM

          DO  30 J30=2,20
            TTEMP(J20*20+J30)=TTEMP(J20*20+1)-1.0*(J30-1)
	    AVAL=TTEMP(J20*20+1)-TTEMP(J20*20+21)
	    BVAL=TTEMP(J20*20+1)-TTEMP(J20*20+J30)
            DELTA=(AVAL-BVAL)*CONS
	    TTIME(J20*20+J30)=TTIME(J20*20+21)+DELTA
30        CONTINUE

20     CONTINUE

C NUMBER POINTS IN FULL TTT:
      ITTTPT=((IPAIRS-1)*20)+1

      ELSEIF (IINFORM.EQ.2) THEN

C INPUT TTT POINTS ARE AT 1 DEGC INTERVALS - THEREFORE NO NEED TO INTERPOLATE
C JUST COPY TTT CURVE WITH 1 DEGC INTERVALS.INTO STORAGE ARRAYS.

       DO 35 I35=1,IPAIRS,1

        TTEMP(I35)=TMTTT(I35)
        TTIME(I35)=TITTT(I35)

35     CONTINUE

C NUMBER POINTS IN 'FULL' TTT:
      ITTTPT=IPAIRS

      ELSE

        WRITE(*,*) 'ERROR - INPUT TTT FORMAT NOT SPECIFIED !'
        PAUSE

      ENDIF


C     WELD COOLING CONSTANTS BRITISH STEEL 1MB DATA, INTERPASS = 20 DEGC
      C1=0.6208D-1
      C2=3.085
      A2=C1*(1-C2)
      A3=1-C2

C CALCULATE STARTEFFHT = EFFECTIVE HEAT INPUT FOR FALL IN TEMP OF 10 DEGC
C FROM THE PARAEQUILIBRIUM AE3 TEMPERATURE IN 50 SECONDS:
      AFAC=(PAE3-20.0)**A3
      BFAC=(PAE3-10.0-20.0)**A3
      STARTEFFHT=50.0*A2/(AFAC-BFAC)
      
C ICCTPT USED TO IN ARRAY THAT STORES CCT CURVE - SET TO ZERO HERE.
      ICCTPT=0

C FIND POINT IN TTT ARRAY WHICH IS BELOW THEQU TEMPERATURE
C WILL IBEGIN SCHEIL'S SUM HERE (SEE INNER LOOP):
        DO 40 J40=1,ITTTPT
	  IF (TTEMP(J40).LT.THEQU) THEN
	    IBEGIN=J40
	    GOTO 50
	  ENDIF
40  	CONTINUE	
      
C OUTER LOOP - COOLING RATE VARIATION MODEL
       
50    DO 80 J80=0,100

C FIRST LOOP ONLY:
      IF (J80.EQ.0) THEN
        IELOOP=150
      ELSE
C TO AVOID REPEATING DATA POINTS - SAVE COMPUTATION TIME:
        IELOOP=29
        NEW1EFFHT=PRE2EFFHT
        NEW2EFFHT=EFFHT
      ENDIF

C MIDDLE LOOP - VARY COOLING RATE.

      DO 70 J70=0,IELOOP

C RESET SUM TO ZERO - TO CALCULATE NEW CCT POINT.
        SUM=0.0

C COOLING RATE VARIATION MODEL:
       
      IF (J80.EQ.0) THEN
C     IE. ON FIRST OUTER LOOP ONLY - REDUCE COMPUATION TIME:
C STORE PREVIOUS EFFECTIVE HEAT INPUTS USED:
        PRE2EFFHT=PREEFFHT
        PREEFFHT=EFFHT
C SET NEW VALUE FOR MORE RAPID COOLING:
        EFFHT=STARTEFFHT*(1/(2**((1.0*J70)*(19.0/150.0))))
      ELSE
C STORE PREVIOUS EFFECTIVE HEAT INPUTS USED:
        PRE2EFFHT=PREEFFHT
        PREEFFHT=EFFHT
C SET NEW VALUE FOR MORE RAPID COOLING:
        EFFHT=NEW1EFFHT-((NEW1EFFHT-NEW2EFFHT)*((1.0*J70)+1)/30.0)
      ENDIF

C     ONLY CALCULATE ISOTHERMAL CONTRIBUTIONS TO THE SCHEILS ADDITION
C     WHEN THE TEMPERTURE FALLS BELOW THE THERMODYNAMIC EQUILBRIUM
C     TEMPERATURE (IE. PAE2,WS,BS AS APPROPRIATE).
   
C     1ST INTERVAL UNIQUE IF THEQU TEMP > TOP TTT CURVE:
       IF ( THEQU.GT.TTEMP(1) ) THEN 
        TINTV=EFFHT*((THEQU-20.0)**A3-(TTEMP(1)-20.0)**A3)/A2
        FRAC=TINTV/TTIME(1)
        SUM=FRAC
       ENDIF
                
C INNER LOOP - APPLY SCHEIL'S REACTION SUM RULE.

C NB. SCHEILS ADDITIVE RACTION RULE ONLY STARTS BELOW THE
C RELEVANT THERMODYNAMIC EQUILIBRIUM TEMPERATURE (THEQU).
C BUT IF THEQU > TOP OF TTT - SEE ABOVE FOR CALC OF IBEGIN

	IENDL=ITTTPT-1     

        DO 60 J60=IBEGIN,IENDL

        TINTV=EFFHT*((TTEMP(J60)-20.0)**A3-(TTEMP(J60+1)-20.0)**A3)/A2

C SCHEIL'S FRACTION CALC - MEAN OF INCUBATION TIMES:

          FRAC=TINTV/((TTIME(J60)+TTIME(J60+1))/2.0) 
          PRETEMP=TEMP
          TEMP=(TTEMP(J60)+TTEMP(J60+1))/2.0	
          PRESUM=SUM
          SUM=SUM+FRAC
         
          IF (SUM.GE.1) THEN

           IF (J60.EQ.IBEGIN) THEN
C SPECIAL CASE - CANNOT INTERPOLATE
            RTEMP=TEMP
           ELSE
C INTERPOLATE:
            CONA=(1.0-PRESUM)/(SUM-PRESUM)
            CONB=(PRETEMP-TEMP)
            RTEMP=PRETEMP-(CONA*CONB)
           ENDIF

C REDUCE DISTORTION AT TOP OF CCT CURVE:
            IF (RTEMP.GT.(TTEMP(1)-5.0)) GOTO 70

C FINISH WHEN TEMP FALLS BELOW MARTENSITE START:
           IF (RTEMP.LT.MS) GOTO 90

C CALCULATES THE TOTAL TIME (REAST) BEFORE THE TRANSFORMATION BEGINS:

C IE THE TIME TAKEN TO COOL FROM THE PARAEQUILIBRIUM AE3 
C TEMPERATURE TO THE TRANSFORMATION START TEMP (RTEMP).

C PAE3 TEMP IS USED FOR THIS PURPOSE REGARDLESS OF THE TRANSFORMATION
C PRODUCT (ALLOTRIOMORPHIC, WIDMANSTATTEN, BAINITIC,OR ACICULAR
C FERRITE) - CCT CURVES ARE CALCULATED RELATIVE TO A FIXED TEMP

       REAST=EFFHT*((PAE3-20.0)**A3-(RTEMP-20.0)**A3)/A2

C STORE CCT POINT:

        ICCTPT=ICCTPT+1
         IF (ICCTPT.GT.1000) THEN
	  WRITE (*,*) "ERROR - CCT ARRAY TOO SMALL !"
          PAUSE 
         ENDIF  
        TMCCT(ICCTPT)=RTEMP
	TICCT(ICCTPT)=REAST
        GOTO 70          
          
       ENDIF

C ALL TTT CURVE USED BUT SUM < 1 GO TO OUTER LOOP:

       IF (J60.EQ.IENDL) GOTO 80

60    CONTINUE

70    CONTINUE

80    CONTINUE

90    RETURN

      END

C*****************************************************************************






